==> nodejs_interop-0.7.0.tar.gz: lib/src/io/file/directory.dart <==
part of nodejs_interop.io;
// ignore_for_file: prefer_constructors_over_static_methods

/// A reference to a directory on the file system.
class Directory extends FileSystemEntity implements f.Directory {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// A reference to the `os` module.
  static final OSModule _os = require('os');

  /// Creates a new directory.
  Directory(String path): super(path);

  /// Creates a new directory from the specified [uri].
  Directory.fromUri(Uri uri): this(uri.toFilePath(windows: const Platform().isWindows));

  /// A [Directory] object pointing to the current working directory.
  static Directory get current => new Directory(process.cwd());

  /// The system temporary directory.
  static Directory get systemTemp => new Directory(_os.tmpdir());

  /// A [Directory] whose path is the absolute path to this entity.
  @override
  Directory get absolute => new Directory(fileSystem.path.absolute(path));

  /// Returns a reference to a [Directory] that exists as a child of this directory and has the specified [basename].
  @override
  Directory childDirectory(String basename) => new Directory(fileSystem.path.join(path, basename));

  /// Returns a reference to a [File] that exists as a child of this directory and has the specified [basename].
  @override
  File childFile(String basename) => new File(fileSystem.path.join(path, basename));

  /// Returns a reference to a [Link] that exists as a child of this directory and has the specified [basename].
  @override
  Link childLink(String basename) => new Link(fileSystem.path.join(path, basename));

  /// Creates the directory with this name.
  /// Uses the [`make-dir`](https://www.npmjs.com/package/make-dir) package if available.
  ///
  /// If [recursive] is `false`, only the last directory in the [path] is created.
  /// If [recursive] is `true`, all non-existing [path] components are created.
  ///
  /// If the directory already exists nothing is done.
  /// If the directory cannot be created the future completes with an exception.
  ///
  /// Returns a [Future] that completes with this [Directory] once it has been created.
  @override
  Future<Directory> create({bool recursive = false}) async {
    if (!recursive) {
      var completer = new Completer<Directory>();
      _fs.mkdir(path, allowInterop(([error]) {
        if (error == null || error.code == SystemError.EEXIST) completer.complete(this);
        else completer.completeError(new f.FileSystemException(error.message, path));
      }));

      return completer.future;
    }

    try {
      var error = new UnsupportedError('Recursivity not supported by Node.js: install the "make-dir" npm package');
      final makeDir = loadLibrary<MakeDirModule>('make-dir', onError: (_) => throw error);
      await convertToFuture(makeDir(path));
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously creates the directory with this name.
  /// Uses the [`make-dir`](https://www.npmjs.com/package/make-dir) package if available.
  ///
  /// If [recursive] is `false`, only the last directory in the [path] is created.
  /// If [recursive] is `true`, all non-existing [path] components are created.
  ///
  /// If the directory already exists nothing is done.
  /// If the directory cannot be created an exception is thrown.
  @override
  void createSync({bool recursive = false}) {
    if (!recursive) {
      try { return _fs.rmdirSync(path); }
      on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
    }

    try {
      var error = new UnsupportedError('Recursivity not supported by Node.js: install the "make-dir" npm package');
      loadLibrary<MakeDirModule>('make-dir', onError: (_) => throw error).sync(path);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Creates a temporary directory in this directory.
  ///
  /// Additional random characters are appended to [prefix] to produce a unique directory name.
  /// If [prefix] is missing or `null`, the empty string is used for [prefix].
  ///
  /// Returns a [Future] that completes with the newly created temporary directory.
  @override
  Future<Directory> createTemp([String prefix]) {
    prefix ??= '';
    var target = prefix.isEmpty ? '${absolute.path}${fileSystem.path.separator}' : fileSystem.path.join(absolute.path, prefix);

    var completer = new Completer<Directory>();
    _fs.mkdtemp(target, allowInterop((error, [folder]) {
      if (error == null) completer.complete(new Directory(folder));
      else completer.completeError(new f.FileSystemException(error.message, target));
    }));

    return completer.future;
  }

  /// Synchronously creates a temporary directory in this directory.
  ///
  /// Additional random characters are appended to [prefix] to produce a unique directory name.
  /// If [prefix] is missing or `null`, the empty string is used for [prefix].
  ///
  /// Returns the newly created temporary directory.
  @override
  Directory createTempSync([String prefix]) {
    prefix ??= '';
    var target = prefix.isEmpty ? '${absolute.path}${fileSystem.path.separator}' : fileSystem.path.join(absolute.path, prefix);
    try { return new Directory(_fs.mkdtempSync(target)); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, target); }
  }

  /// Deletes this directory.
  /// Uses the [`del`](https://www.npmjs.com/package/del) package if available.
  ///
  /// If the [FileSystemEntity] is a directory, and if [recursive] is `false`, the directory must be empty.
  /// Otherwise, if [recursive] is `true`, the directory and all sub-directories and files in the directories are deleted.
  /// Links are not followed when deleting recursively. Only the link is deleted, not its target.
  ///
  /// Returns a [Future] that completes with this [FileSystemEntity] when the deletion is done.
  /// If the [FileSystemEntity] cannot be deleted, the future completes with an exception.
  @override
  Future<Directory> delete({bool recursive = false}) async {
    if (!recursive) {
      var completer = new Completer<Directory>();
      _fs.rmdir(path, allowInterop(([error]) {
        if (error == null) completer.complete(this);
        else completer.completeError(new f.FileSystemException(error.message, path));
      }));

      return completer.future;
    }

    try {
      var error = new UnsupportedError('Recursivity not supported by Node.js: install the "del" npm package');
      final del = loadLibrary<DelModule>('del', onError: (_) => throw error);
      await convertToFuture(del(fileSystem.path.join(path, '**').replaceAll(r'\', '/')));
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously deletes this directory.
  /// Uses the [`del`](https://www.npmjs.com/package/del) package if available.
  ///
  /// If the [FileSystemEntity] is a directory, and if [recursive] is `false`, the directory must be empty.
  /// Otherwise, if [recursive] is `true`, the directory and all sub-directories and files in the directories are deleted.
  /// Links are not followed when deleting recursively. Only the link is deleted, not its target.
  ///
  /// Throws an exception if the [FileSystemEntity] cannot be deleted.
  @override
  void deleteSync({bool recursive = false}) {
    if (!recursive) {
      try { return _fs.rmdirSync(path); }
      on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
    }

    try {
      var error = new UnsupportedError('Recursivity not supported by Node.js: install the "del" npm package');
      loadLibrary<DelModule>('del', onError: (_) => throw error).sync(fileSystem.path.join(path, '**').replaceAll(r'\', '/'));
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Lists the sub-directories and files of this directory. Optionally recurses into sub-directories.
  /// Uses the [`readdir-enhanced`](https://www.npmjs.com/package/readdir-enhanced) package if available.
  ///
  /// If [followLinks] is `false`, then any symbolic links found are reported as [Link] objects,
  /// rather than as directories or files, and are not recursed into.
  ///
  /// If [followLinks] is `true`, then working links are reported as directories or files, depending on their type,
  /// and links to directories are recursed into. Broken links are reported as [Link] objects.
  /// If a symbolic link makes a loop in the file system, then a recursive listing will not follow
  /// a link twice in the same recursive descent, but will report it as a [Link] the second time it is seen.
  ///
  /// The result is a stream of [FileSystemEntity] objects for the directories, files, and links.
  @override
  Stream<FileSystemEntity> list({bool followLinks = true, bool recursive = false}) {
    var controller = new StreamController<FileSystemEntity>();

    if (!recursive) _fs.readdir(path, allowInterop((error, [paths]) async {
      if (error != null) controller.addError(new f.FileSystemException(error.message, path));
      else for (var item in paths) controller.add(_toFileSystemEntity(item, await FileSystemEntity.type(item, followLinks: followLinks)));
      return controller.close();
    }));
    else {
      var error = new UnsupportedError('Recursivity not supported by Node.js: install the "readdir-enhanced" npm package');
      loadLibrary<ReaddirModule>('readdir-enhanced', onError: (_) => throw error).stream(path, new ReaddirOptions(deep: true))
        .on('error', allowInterop((error) => controller..addError(new f.FileSystemException(error.message))..close()))
        .on('directory', allowInterop((path) => controller.add(new Directory(path))))
        .on('file', allowInterop((path) => controller.add(new File(path))))
        .on('symlink', allowInterop((path) => controller.add(new Link(path))))
        .on('end', allowInterop((path) => controller.close()));
    }

    return controller.stream;
  }

  /// Synchronously lists the sub-directories and files of this directory. Optionally recurses into sub-directories.
  /// Uses the [`readdir-enhanced`](https://www.npmjs.com/package/readdir-enhanced) package if available.
  ///
  /// The result is a list of [FileSystemEntity] objects for the directories, files, and links.
  @override
  List<FileSystemEntity> listSync({bool followLinks = true, bool recursive = false}) {
    try {
      List<String> paths;
      if (!recursive) paths = _fs.readdirSync(path);
      else {
        var error = new UnsupportedError('Recursivity not supported by Node.js: install the "readdir-enhanced" npm package');
        paths = loadLibrary<ReaddirModule>('readdir-enhanced', onError: (_) => throw error).sync(path, new ReaddirOptions(deep: true));
      }

      return paths.map((item) => _toFileSystemEntity(item, FileSystemEntity.typeSync(item, followLinks: followLinks))).toList();
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Renames this directory. Completes with a [Directory] instance for the renamed directory.
  @override
  Future<Directory> rename(String newPath) {
    var completer = new Completer<Directory>();
    _fs.rename(path, newPath, allowInterop(([error]) {
      if (error == null) completer.complete(new Directory(newPath));
      else completer.completeError(new f.FileSystemException(error.message, newPath));
    }));

    return completer.future;
  }

  /// Synchronously renames this directory. Returns a [Directory] instance for the renamed directory.
  @override
  Directory renameSync(String newPath) {
    try {
      _fs.renameSync(path, newPath);
      return new Directory(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Creates a [FileSystemEntity] of the specified [type] for the given [path].
  FileSystemEntity _toFileSystemEntity(String path, f.FileSystemEntityType type) {
    switch (type) {
      case f.FileSystemEntityType.DIRECTORY: return new Directory(path);
      case f.FileSystemEntityType.FILE: return new File(path);
      case f.FileSystemEntityType.LINK: return new Link(path);
      default: throw new f.FileSystemException('Unknown file system entity', path);
    }
  }
}

==> nodejs_interop-0.8.0.tar.gz: lib/src/io/file/directory.dart <==
part of nodejs_interop.io;

/// A reference to a directory on the file system.
class Directory extends FileSystemEntity implements f.Directory {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// A reference to the `os` module.
  static final OSModule _os = require('os');

  /// Creates a new directory.
  Directory(String path): super._internal(path);

  /// Creates a new directory from the specified [uri].
  Directory.fromUri(Uri uri): this(uri.toFilePath(windows: const Platform().isWindows));

  /// A [Directory] object pointing to the current working directory.
  static Directory get current => new Directory(process.cwd()); // ignore: prefer_constructors_over_static_methods
  static set current(Object value) => process.chdir(value is f.Directory ? value.path : value.toString());

  /// The system temporary directory.
  static Directory get systemTemp => new Directory(_os.tmpdir()); // ignore: prefer_constructors_over_static_methods

  /// A [Directory] whose path is the absolute path to this entity.
  @override
  Directory get absolute => new Directory(fileSystem.path.absolute(path));

  /// A [Uri] representing the entity's location.
  @override
  Uri get uri => new Uri.directory(path, windows: const Platform().isWindows);

  /// Returns a reference to a [Directory] that exists as a child of this directory and has the specified [basename].
  @override
  Directory childDirectory(String basename) => new Directory(fileSystem.path.join(path, basename));

  /// Returns a reference to a [File] that exists as a child of this directory and has the specified [basename].
  @override
  File childFile(String basename) => new File(fileSystem.path.join(path, basename));

  /// Returns a reference to a [Link] that exists as a child of this directory and has the specified [basename].
  @override
  Link childLink(String basename) => new Link(fileSystem.path.join(path, basename));

  /// Creates the directory with this name.
  /// Uses the [`make-dir`](https://www.npmjs.com/package/make-dir) package if available.
  ///
  /// If [recursive] is `false`, only the last directory in the [path] is created.
  /// If [recursive] is `true`, all non-existing [path] components are created.
  ///
  /// If the directory already exists nothing is done.
  /// If the directory cannot be created the future completes with an exception.
  ///
  /// Returns a [Future] that completes with this [Directory] once it has been created.
  @override
  Future<Directory> create({bool recursive = false}) async {
    try {
      await (!recursive ? fn1cb0ToFuture(_fs.mkdir, path) : asFuture(loadLibrary<MakeDirModule>('make-dir', onError: (_) =>
        throw new UnsupportedError('Not supported by Node.js: install the "make-dir" npm package')
      ).call(path)));

      return this;
    }

    on JSError catch (error) {
      if (error.code == SystemError.EEXIST) return this;
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously creates the directory with this name.
  /// Uses the [`make-dir`](https://www.npmjs.com/package/make-dir) package if available.
  ///
  /// If [recursive] is `false`, only the last directory in the [path] is created.
  /// If [recursive] is `true`, all non-existing [path] components are created.
  ///
  /// If the directory already exists nothing is done.
  /// If the directory cannot be created an exception is thrown.
  @override
  void createSync({bool recursive = false}) {
    try {
      if (!recursive) _fs.mkdirSync(path);
      else loadLibrary<MakeDirModule>('make-dir', onError: (_) =>
        throw new UnsupportedError('Not supported by Node.js: install the "make-dir" npm package')
      ).sync(path);
    }

    on JSError catch (error) {
      if (error.code != SystemError.EEXIST) throw new f.FileSystemException(error.message, path);
    }
  }

  /// Creates a temporary directory in this directory.
  ///
  /// Additional random characters are appended to [prefix] to produce a unique directory name.
  /// If [prefix] is missing or `null`, the empty string is used for [prefix].
  ///
  /// Returns a [Future] that completes with the newly created temporary directory.
  @override
  Future<Directory> createTemp([String prefix]) async {
    prefix ??= '';
    var target = prefix.isEmpty ? '${absolute.path}${fileSystem.path.separator}' : fileSystem.path.join(absolute.path, prefix);
    try { return new Directory(await fn1cb1ToFuture<String>(_fs.mkdtemp, target)); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, target); }
  }

  /// Synchronously creates a temporary directory in this directory.
  ///
  /// Additional random characters are appended to [prefix] to produce a unique directory name.
  /// If [prefix] is missing or `null`, the empty string is used for [prefix].
  ///
  /// Returns the newly created temporary directory.
  @override
  Directory createTempSync([String prefix]) {
    prefix ??= '';
    var target = prefix.isEmpty ? '${absolute.path}${fileSystem.path.separator}' : fileSystem.path.join(absolute.path, prefix);
    try { return new Directory(_fs.mkdtempSync(target)); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, target); }
  }

  /// Deletes this directory.
  /// Uses the [`del`](https://www.npmjs.com/package/del) package if available.
  ///
  /// If the [FileSystemEntity] is a directory, and if [recursive] is `false`, the directory must be empty.
  /// Otherwise, if [recursive] is `true`, the directory and all sub-directories and files in the directories are deleted.
  /// Links are not followed when deleting recursively. Only the link is deleted, not its target.
  ///
  /// Returns a [Future] that completes with this [FileSystemEntity] when the deletion is done.
  /// If the [FileSystemEntity] cannot be deleted, the future completes with an exception.
  @override
  Future<Directory> delete({bool recursive = false}) async {
    try {
      await (!recursive ? fn1cb0ToFuture(_fs.rmdir, path) : asFuture(loadLibrary<DelModule>('del', onError: (_) =>
        throw new UnsupportedError('Not supported by Node.js: install the "del" npm package')
      ).call(fileSystem.path.join(path, '**').replaceAll(r'\', '/'))));

      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously deletes this directory.
  /// Uses the [`del`](https://www.npmjs.com/package/del) package if available.
  ///
  /// If the [FileSystemEntity] is a directory, and if [recursive] is `false`, the directory must be empty.
  /// Otherwise, if [recursive] is `true`, the directory and all sub-directories and files in the directories are deleted.
  /// Links are not followed when deleting recursively. Only the link is deleted, not its target.
  ///
  /// Throws an exception if the [FileSystemEntity] cannot be deleted.
  @override
  void deleteSync({bool recursive = false}) {
    try {
      if (!recursive) _fs.rmdirSync(path);
      else loadLibrary<DelModule>('del', onError: (_) =>
        throw new UnsupportedError('Not supported by Node.js: install the "del" npm package')
      ).sync(fileSystem.path.join(path, '**').replaceAll(r'\', '/'));
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Lists the sub-directories and files of this directory. Optionally recurses into sub-directories.
  /// Uses the [`readdir-enhanced`](https://www.npmjs.com/package/readdir-enhanced) package if available.
  ///
  /// If [followLinks] is `false`, then any symbolic links found are reported as [Link] objects,
  /// rather than as directories or files, and are not recursed into.
  ///
  /// If [followLinks] is `true`, then working links are reported as directories or files, depending on their type,
  /// and links to directories are recursed into. Broken links are reported as [Link] objects.
  /// If a symbolic link makes a loop in the file system, then a recursive listing will not follow
  /// a link twice in the same recursive descent, but will report it as a [Link] the second time it is seen.
  ///
  /// The result is a stream of [FileSystemEntity] objects for the directories, files, and links.
  @override
  Stream<FileSystemEntity> list({bool followLinks = true, bool recursive = false}) {
    var controller = new StreamController<FileSystemEntity>();

    if (!recursive) _fs.readdir(path, allowInterop((error, [paths]) async {
      if (error != null)
        controller.addError(new f.FileSystemException(error.message, path));
      else for (var item in paths)
        controller.add(_toFileSystemEntity(item, await FileSystemEntity.type(item, followLinks: followLinks))); // ignore: avoid_slow_async_io

      await controller.close();
    }));
    else {
      loadLibrary<ReaddirModule>('readdir-enhanced', onError: (_) =>
        throw new UnsupportedError('Not supported by Node.js: install the "readdir-enhanced" npm package')
      ).stream(path, new ReaddirOptions(deep: true))
        .on('error', allowInterop((error) => controller..addError(new f.FileSystemException(error.message))..close()))
        .on('directory', allowInterop((path) => controller.add(new Directory(path))))
        .on('file', allowInterop((path) => controller.add(new File(path))))
        .on('symlink', allowInterop((path) => controller.add(new Link(path))))
        .on('end', allowInterop((path) => controller.close()));
    }

    return controller.stream;
  }

  /// Synchronously lists the sub-directories and files of this directory. Optionally recurses into sub-directories.
  /// Uses the [`readdir-enhanced`](https://www.npmjs.com/package/readdir-enhanced) package if available.
  ///
  /// The result is a list of [FileSystemEntity] objects for the directories, files, and links.
  @override
  List<FileSystemEntity> listSync({bool followLinks = true, bool recursive = false}) {
    try {
      var paths = !recursive ? _fs.readdirSync(path) : loadLibrary<ReaddirModule>('readdir-enhanced', onError: (_) =>
        throw new UnsupportedError('Not supported by Node.js: install the "readdir-enhanced" npm package')
      ).sync(path, new ReaddirOptions(deep: true));

      return paths.map((item) => _toFileSystemEntity(item, FileSystemEntity.typeSync(item, followLinks: followLinks))).toList();
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Renames this directory. Completes with a [Directory] instance for the renamed directory.
  @override
  Future<Directory> rename(String newPath) async {
    try {
      await fn2cb0ToFuture(_fs.rename, path, newPath);
      return new Directory(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Synchronously renames this directory. Returns a [Directory] instance for the renamed directory.
  @override
  Directory renameSync(String newPath) {
    try {
      _fs.renameSync(path, newPath);
      return new Directory(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Returns a string representation of this object.
  @override
  String toString() => 'Directory("$path")';

  /// Creates a [FileSystemEntity] of the specified [type] for the given [path].
  FileSystemEntity _toFileSystemEntity(String path, f.FileSystemEntityType type) {
    switch (type) {
      case f.FileSystemEntityType.DIRECTORY: return new Directory(path);
      case f.FileSystemEntityType.FILE: return new File(path);
      case f.FileSystemEntityType.LINK: return new Link(path);
      default: throw new f.FileSystemException('Unknown file system entity', path);
    }
  }
}
